/*
 * Copyright (c) 2004-2006 Extreme! Lab, Indiana University. All rights reserved.
 * 
 * This software is open source. See the bottom of this file for the license.
 * 
 * $Id: EdgeImpl.java,v 1.5 2006/11/20 18:35:16 sshirasu Exp $
 */

package edu.indiana.extreme.xbaya.graph.impl;

import org.xmlpull.infoset.XmlElement;

import edu.indiana.extreme.xbaya.graph.Edge;
import edu.indiana.extreme.xbaya.graph.GraphException;
import edu.indiana.extreme.xbaya.graph.GraphSchema;
import edu.indiana.extreme.xbaya.graph.gui.EdgeGUI;
import edu.indiana.extreme.xbaya.util.XMLUtil;

/**
 * The Edge represents an edge that connects a uses port to a provide port.
 * 
 * @author Satoshi Shirasuna
 */
public abstract class EdgeImpl implements Edge {

    private GraphImpl graph;

    private PortImpl fromPort;

    private PortImpl toPort;

    private EdgeGUI gui;

    // The followings are used only during parsing the XML.

    private String fromPortID;

    private String toPortID;

    /**
     * Creates an Edge.
     */
    public EdgeImpl() {
        // Do nothing
    }

    /**
     * Constructs a EdgeImpl.
     * 
     * @param edgeXml
     */
    public EdgeImpl(XmlElement edgeXml) {
        parse(edgeXml);
    }

    /**
     * @see edu.indiana.extreme.xbaya.graph.Edge#getGUI()
     */
    public EdgeGUI getGUI() {
        if (this.gui == null) {
            this.gui = new EdgeGUI(this);
        }
        return this.gui;
    }

    /**
     * @see edu.indiana.extreme.xbaya.graph.Edge#getFromPort()
     */
    public PortImpl getFromPort() {
        return this.fromPort;
    }

    /**
     * @see edu.indiana.extreme.xbaya.graph.Edge#getToPort()
     */
    public PortImpl getToPort() {
        return this.toPort;
    }

    /**
     * Sets a graph this edge belogs to.
     * 
     * @param graph
     *            The graph
     */
    protected void setGraph(GraphImpl graph) {
        this.graph = graph;
    }

    /**
     * @param fromPort
     */
    protected void setFromPort(PortImpl fromPort) {
        this.fromPort = fromPort;
    }

    /**
     * @param toPort
     */
    protected void setToPort(PortImpl toPort) {
        this.toPort = toPort;
    }

    protected void indexToPointer() throws GraphException {
        this.fromPort = this.graph.getPort(this.fromPortID);
        if (this.fromPort == null) {
            throw new GraphException("Cannot find a port with the ID, "
                    + this.fromPortID + ".");
        }
        this.toPort = this.graph.getPort(this.toPortID);
        if (this.toPort == null) {
            throw new GraphException("Cannot find a port with the ID, "
                    + this.toPortID + ".");
        }

        // Has to do the above first because they are used in the edgeWasAdded
        // method.
        this.fromPort.addEdge(this);
        this.toPort.addEdge(this);
    }

    /**
     * @param edgeElement
     */
    protected void parse(XmlElement edgeElement) {
        XmlElement fromPortElement = edgeElement
                .element(GraphSchema.EDGE_FROM_PORT_TAG);
        this.fromPortID = fromPortElement.requiredText();

        XmlElement toPortElement = edgeElement
                .element(GraphSchema.EDGE_TO_PORT_TAG);
        this.toPortID = toPortElement.requiredText();
    }

    /**
     * @return the XmlElement
     */
    protected XmlElement toXML() {
        XmlElement edgeXml = XMLUtil.BUILDER.newFragment(GraphSchema.NS,
                GraphSchema.EDGE_TAG);

        XmlElement fromEle = edgeXml.addElement(GraphSchema.NS,
                GraphSchema.EDGE_FROM_PORT_TAG);
        fromEle.addChild(this.fromPort.getID());

        XmlElement toEle = edgeXml.addElement(GraphSchema.NS,
                GraphSchema.EDGE_TO_PORT_TAG);
        toEle.addChild(this.toPort.getID());

        return edgeXml;
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2004-2006 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */